/***************************************************
  This is an example sketch for our optical Fingerprint sensor

  Designed specifically to work with the Adafruit BMP085 Breakout
  ----> http://www.adafruit.com/products/751

  These displays use TTL Serial to communicate, 2 pins are required to
  interface
  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ****************************************************/

#include <Adafruit_Fingerprint.h>

// On Leonardo/Micro or others with hardware serial, use those! #0 is green wire, #1 is white
// uncomment this line:
// #define mySerial Serial1

// For UNO and others without hardware serial, we must use software serial...
// pin #2 is IN from sensor (GREEN wire)
// pin #3 is OUT from arduino  (WHITE wire)
// comment these two lines if using hardware serial
SoftwareSerial mySerial(2, 3);

Adafruit_Fingerprint finger = Adafruit_Fingerprint(&mySerial);

// include the library code FOR i2c LCD:
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal_I2C lcd(0x27, 16, 2);

String upperRow;
String lowerRow;
boolean refreshLCD = false;
long defLCD = 0;
#define REFRESH     5000

uint16_t MASTER_ID=1;

#define FINGER_FOUND   546
#define FINGER_ENROLLED  857
#define FINGER_NO_FOUND  457

uint16_t id;
uint16_t rezultat, foundID;
int oMaster, uoMaster, change=1;

void writeToLCD(String upRow, String lwRow) {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(upRow);
  lcd.setCursor(0, 1);
  lcd.print(lwRow);
  Serial.println(upRow);
  Serial.println(lwRow);
}

uint16_t getFingerprintEnroll() {
  int p = -1;
  
  p = finger.getImage();
  switch (p) {
    case FINGERPRINT_OK:
       Serial.println("Image taken");
       break;
    case FINGERPRINT_NOFINGER:
       //Serial.println(".");
       return(p);
    case FINGERPRINT_PACKETRECIEVEERR:
       Serial.println("Communication error");
       return(p);
    case FINGERPRINT_IMAGEFAIL:
       Serial.println("Imaging error");
       return(p);
    default:
       Serial.println("Unknown error");
       return(p);
  }

  // OK success!
  p = finger.image2Tz(1);
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("Image converted");
      break;
    case FINGERPRINT_IMAGEMESS:
      Serial.println("Image too messy");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("Communication error");
      return p;
    case FINGERPRINT_FEATUREFAIL:
      Serial.println("Could not find fingerprint features");
      return p;
    case FINGERPRINT_INVALIDIMAGE:
      Serial.println("Could not find fingerprint features");
      return p;
    default:
      Serial.println("Unknown error");
      return p;
  }

  // OK converted!
  Serial.print("Searching in reader memory #");
  p = finger.fingerSearch();
  if (p == FINGERPRINT_OK) {
    foundID=finger.fingerID;
    Serial.print("Found a print match at: ");
    Serial.println(foundID);
    Serial.print(" with confidence of "); Serial.println(finger.confidence);
    return FINGER_FOUND;
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("Communication error");
    return p;
  } else if (p == FINGERPRINT_NOTFOUND) {
    Serial.println("Did not find a match");
  } else {
    Serial.println("Unknown error");
    return p;
  }

  Serial.println("Remove finger");
  upperRow=String(F("UPISIVANJE #"))+String(id-1);
  lowerRow = String(F("Remove finger"));
  writeToLCD(upperRow, lowerRow);   
  delay(2000);
  p = 0;
  while (p != FINGERPRINT_NOFINGER) {
    p = finger.getImage();
  }
  Serial.print("ID "); Serial.println(id);
  p = -1;
  Serial.println("Place same finger again");
  upperRow=String(F("UPISIVANJE #"))+String(id-1);
  lowerRow = String(F("Place again!"));
  writeToLCD(upperRow, lowerRow);     
  while (p != FINGERPRINT_OK) {
    p = finger.getImage();
    switch (p) {
      case FINGERPRINT_OK:
        Serial.println("Image taken");
        break;
      case FINGERPRINT_NOFINGER:
        //Serial.print(".");
        break;
      case FINGERPRINT_PACKETRECIEVEERR:
        Serial.println("Communication error");
        break;
      case FINGERPRINT_IMAGEFAIL:
        Serial.println("Imaging error");
        break;
      default:
        Serial.println("Unknown error");
        break;
    }
  }

  // OK success!

  p = finger.image2Tz(2);
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("Image converted");
      break;
    case FINGERPRINT_IMAGEMESS:
      Serial.println("Image too messy");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("Communication error");
      return p;
    case FINGERPRINT_FEATUREFAIL:
      Serial.println("Could not find fingerprint features");
      return p;
    case FINGERPRINT_INVALIDIMAGE:
      Serial.println("Could not find fingerprint features");
      return p;
    default:
      Serial.println("Unknown error");
      return p;
  }

  // OK converted!
  Serial.print("Creating model for #");  Serial.println(id);
  p = finger.createModel();
  if (p == FINGERPRINT_OK) {
    Serial.println("Prints matched!");
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("Communication error");
    return p;
  } else if (p == FINGERPRINT_ENROLLMISMATCH) {
    Serial.println("Fingerprints did not match");
    return p;
  } else {
    Serial.println("Unknown error");
    return p;
  }

  Serial.print("ID "); Serial.println(id);
  p = finger.storeModel(id);
  Serial.print("Store model "); Serial.println(id);
  if (p == FINGERPRINT_OK) {
    Serial.println("Stored!");
    return FINGER_ENROLLED;
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("Communication error");
    return p;
  } else if (p == FINGERPRINT_BADLOCATION) {
    Serial.println("Could not store in that location");
    return p;
  } else if (p == FINGERPRINT_FLASHERR) {
    Serial.println("Error writing to flash");
    return p;
  } else {
    Serial.println("Unknown error");
    return p;
  }
}


uint16_t getFingerprintID() {
  uint8_t p = finger.getImage();
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("Image taken");
      break;
    case FINGERPRINT_NOFINGER:
      //Serial.println("No finger detected");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("Communication error");
      return p;
    case FINGERPRINT_IMAGEFAIL:
      Serial.println("Imaging error");
      return p;
    default:
      Serial.println("Unknown error");
      return p;
  }

  // OK success!

  p = finger.image2Tz();
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("Image converted");
      break;
    case FINGERPRINT_IMAGEMESS:
      Serial.println("Image too messy");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("Communication error");
      return p;
    case FINGERPRINT_FEATUREFAIL:
      Serial.println("Could not find fingerprint features");
      return p;
    case FINGERPRINT_INVALIDIMAGE:
      Serial.println("Could not find fingerprint features");
      return p;
    default:
      Serial.println("Unknown error");
      return p;
  }

  // OK converted!
  p = finger.fingerSearch();
  if (p == FINGERPRINT_OK) {
    Serial.println("Found a print match!");
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("Communication error");
    return p;
  } else if (p == FINGERPRINT_NOTFOUND) {
    Serial.println("Did not find a match");
    return FINGER_NO_FOUND;
  } else {
    Serial.println("Unknown error");
    return p;
  }

  // found a match!
  foundID=finger.fingerID;
  Serial.print("Found ID #"); Serial.print(foundID);
  Serial.print(" with confidence of "); Serial.println(finger.confidence);

  return FINGER_FOUND;
}


void setup()
{
  Serial.begin(9600);
  while (!Serial);  // For Yun/Leo/Micro/Zero/...
  delay(100);
  Serial.println("\n\nAdafruit Fingerprint sensor enrollment");

  // Inicijalizacija LCD-a
  lcd.init();
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  //To power on backlight
  lcd.backlight();

  // set the data rate for the sensor serial port
  finger.begin(57600);  //57600

  if (finger.verifyPassword()) {
    Serial.println("Found fingerprint sensor!");
  } else {
    Serial.println("Did not find fingerprint sensor :(");
    while (1) {
      delay(1);
    }
  }
  Serial.println(F("Reading sensor parameters"));
  finger.getParameters();
  Serial.print(F("Status: 0x")); Serial.println(finger.status_reg, HEX);
  Serial.print(F("Sys ID: 0x")); Serial.println(finger.system_id, HEX);
  Serial.print(F("Capacity: ")); Serial.println(finger.capacity);
  Serial.print(F("Security level: ")); Serial.println(finger.security_level);
  Serial.print(F("Device address: ")); Serial.println(finger.device_addr, HEX);
  Serial.print(F("Packet len: ")); Serial.println(finger.packet_len);
  Serial.print(F("Baud rate: ")); Serial.println(finger.baud_rate);

  finger.getTemplateCount();
  id=finger.templateCount;
  Serial.print("Sensor contains "); Serial.print(finger.templateCount); Serial.println(" templates"); 
  if (id == 0) {
    Serial.print("Sensor doesn't contain any fingerprint data. Please run the 'enroll' example.");
  }
  id++;  
  Serial.println("Waiting for valid finger...");

  upperRow=String(F("UPISIVANJE #"))+String(id-1);
  lowerRow = String(F(""));
  writeToLCD(upperRow, lowerRow);    
}


void loop()                     // run over and over again
{
  //Provjera uslova za brisanje baze čitača
  if(uoMaster==5){
    // Ovdje dopunite kod
    finger.getTemplateCount();
    id=finger.templateCount;
    id++;
  }
  //Provjera uslova za rad u mod prepoznavanja
  else if(oMaster%2){
    rezultat=getFingerprintID();
    //Provjera da li je očitani otisak pronađen u memoriji čitača
    if(rezultat==FINGER_FOUND){
      //Provjera da li je prepoznat MASTER otisak
      if(MASTER_ID == foundID){
          // Ovdje dopunite kod  
      }
      //Ili je prepoznat drugi otisak
      else{
          // Ovdje dopunite kod
      }
    }
    //Očitanog otiska nema u memoriji čitača
    else if (rezultat==FINGER_NO_FOUND){
      // Ovdje dopunite kod      
    }
  }
  //Provjera uslova za rad u modu upisivanja
  else if(oMaster%2==0){
    rezultat=getFingerprintEnroll();
    //Provjera da li je očitani otisak pronađen u memoriji čitača
    if(rezultat==FINGER_FOUND){
      //Provjera da li je prepoznat MASTER otisak
      if(MASTER_ID == foundID){
        // Ovdje dopunite kod    
      }
      //Ili je prepoznat drugi otisak
      else{
         // Ovdje dopunite kod
      }
    }
    //Provjera da li je očitani otisak upisan
    else if(rezultat==FINGER_ENROLLED){    
        // Ovdje dopunite kod
    }
    //Ukoliko upisivanje nije uspjelo a otisak je bio na čitaču
    else if (rezultat!=FINGERPRINT_NOFINGER){
      // Ovdje dopunite kod        
    }   
  }

  //Provjera treba li vratiti default ispis
  if ( ((millis() - defLCD) > REFRESH) && defLCD)  {
    if (oMaster%2)upperRow = String(F("PREPOZNAVA  #"))+String(id-1);
    else upperRow = String(F("UPISIVANJE!  #"))+String(id-1);
    lowerRow = String(F(""));
    defLCD = 0;
    refreshLCD = true;
  }

  //Ispis na LCD-u
  if (refreshLCD) {
    writeToLCD(upperRow, lowerRow);
    refreshLCD = false;
  }
}
